body.app-log_outgoing_requests.model-outgoingrequestslog .field-url .readonly,
body.app-log_outgoing_requests.model-outgoingrequestslog .field-method .readonly,
body.app-log_outgoing_requests.model-outgoingrequestslog .field-status_code .readonly {
  font-weight: 900;
}

body.app-log_outgoing_requests.model-outgoingrequestslog .field-request_body .readonly,
body.app-log_outgoing_requests.model-outgoingrequestslog .field-req_headers .readonly,
body.app-log_outgoing_requests.model-outgoingrequestslog .field-response_body .readonly,
body.app-log_outgoing_requests.model-outgoingrequestslog .field-res_headers .readonly {
  white-space: pre-wrap;
  font-family: monospace;
  overflow: auto;

}
