# Generated by Django 4.1 on 2023-06-08 13:52

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("log_outgoing_requests", "0002_outgoingrequestslogconfig_and_more"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="outgoingrequestslog",
            options={
                "verbose_name": "Outgoing request log",
                "verbose_name_plural": "Outgoing request logs",
            },
        ),
        migrations.AlterModelOptions(
            name="outgoingrequestslogconfig",
            options={"verbose_name": "Outgoing request log configuration"},
        ),
        migrations.AlterField(
            model_name="outgoingrequestslog",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="outgoingrequestslogconfig",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
    ]
